@@ -1,13 +1,13 @@
 //! Provides methods to manipulate the ECUs diagnostic session mode
 
-use crate::DiagServerResult;
+use crate::{DiagServerResult, DiagnosticServer};
 
 use super::{KWP2000Command, Kwp2000DiagnosticServer};
 
 /// KWP2000 diagnostic session type
-/// 
+///
 /// Session support matrix
-/// 
+///
 /// | SessionType | Support by ECUs |
 /// |--|--|
 /// |[SessionType::Normal] | Mandatory |
@@ -58,6 +58,11 @@ impl From<SessionType> for u8 {
 /// ## Parameters
 /// * server - The KWP2000 Diagnostic server
 /// * mode - The [Kwp2000SessionType] to put the ECU into
-pub fn set_diagnostic_session_mode(server: &mut Kwp2000DiagnosticServer, mode: SessionType) -> DiagServerResult<()> {
-    server.execute_command(KWP2000Command::StartDiagnosticSession, &[mode.into()]).map(|_| ())
+pub fn set_diagnostic_session_mode(
+    server: &mut Kwp2000DiagnosticServer,
+    mode: SessionType,
+) -> DiagServerResult<()> {
+    server
+        .execute_command_with_response(KWP2000Command::StartDiagnosticSession, &[mode.into()])
+        .map(|_| ())
 }
